
-------------------------------------------------------------------------------
-- Module Declaration
--

local mod, CL = BigWigs:NewBoss("Blackhand Assist", 988)
if not mod then return end
mod:RegisterEnableMob(77325) -- 87318 = Garrison Training Dummy
mod.engageId = 1704

local addon = CreateFrame("Frame", "BlackhandAssist")

-------------------------------------------------------------------------------
-- Locals
--

local icons = {
	"|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_1.blp:0|t",
	"|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_2.blp:0|t",
	"|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_3.blp:0|t",
	"|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_4.blp:0|t",
	"|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_5.blp:0|t",
	"|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_6.blp:0|t",
	"|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_7.blp:0|t",
	"|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8.blp:0|t",
}

local colors = {
	{1.0, 0.92, 0},
	{0.98, 0.57, 0},
	{0.83, 0.22, 0.9},
	{0.04, 0.95, 0},
	{0.7, 0.82, 0.875},
	{0, 0.71, 1},
	{1.0, 0.24, 0.168},
	{0.98, 0.98, 0.98},
}

local smashTargetsLeft = {
	{"1Hans"},
	{"2Peter"},
	{"3Fritz"},
	{"4Hans"},
	{"5Peter"},
}
local smashTargetsRight = {
	{"1Hans", "Elvador"},
	{"2Peter"},
	{"3Fritz"},
	{"4Hans"},
	{"5Peter"},
}

local phase = nil
local smashCount = nil
local mortarWarnings = {}
local markCount = nil
local markOnMe = nil
local myMarkPosition = nil
local myBombPosition = nil

-------------------------------------------------------------------------------
-- Locale
--
local L = mod:NewLocale("enUS", true)
if L then
	L.modname = "Blackhand Assist"

	L.assist = "Assist"
	L.assist_desc = "Open the other config for more options /bwblackhand (Assuming the module is enabled (to enable the module, mouse over the boss))."

	L.left = "Left"
	L.right = "Right"
	L.phase = "Phase %d"
	L.custom_bombs = "Custom Bombs say"
	L.custom_marks = "Custom Marked for Death say"
end
L = mod:GetLocale()
mod.displayName = L.modname

--------------------------------------------------------------------------------
-- Initialization
--

function mod:GetOptions()
	return {
		156530,
		156425,
		{156096, "SAY"},
		"assist",
	}
end

-------------------------------------------------------------------------------
-- Options
--

local defaults = {
	profile = {
		enabled = true,
		lefticon = 4,
		righticon = 6,
		bombsenable = true,
		markedenable = true,
	}
}

local values = {
	RAID_TARGET_1,
	RAID_TARGET_2,
	RAID_TARGET_3,
	RAID_TARGET_4,
	RAID_TARGET_5,
	RAID_TARGET_6,
	RAID_TARGET_7,
	RAID_TARGET_8,
}

local disabled = function() return not addon.db.profile.enabled end

local options = {
	name = L.modname,
	handler = addon,
	type = "group",
	get = function(info) return addon.db.profile[info[#info]] end,
	set = function(info, value) addon.db.profile[info[#info]] = value end,
	args = {
		enabled = {
			type = "toggle",
			name = ENABLE,
			order = 2,
		},
		p1_header = {
			type = "header",
			name = L.phase:format(1),
			order = 100,
		},
		p2_header = {
			type = "header",
			name = L.phase:format(2),
			order = 200,
		},
		lefticon = {
			type = "select",
			name = L.left,
			values = values,
			itemControl = "DDI-RaidIcon",
			disabled = disabled,
			order = 201,
		},
		righticon = {
			type = "select",
			name = L.right,
			values = values,
			itemControl = "DDI-RaidIcon",
			disabled = disabled,
			order = 202,
		},
		smash_player_left = {
			type = "description",
			name = "Smash Players Left: ",
			disabled = disabled,
			width = "double",
			order = 210,
		},
		smash_player_left1 = {
			type = "description",
			name = "1. Player1, Player2, ...",
			disabled = disabled,
			width = "double",
			order = 211,
		},
		smash_player_left_add1 = {
			type = "input",
			name = "Add player smash left 1",
			disabled = disabled,
			order = 212,
		},
		smash_player_left2 = {
			type = "description",
			name = "2. Player1, Player2, ...",
			disabled = disabled,
			width = "double",
			order = 221,
		},
		smash_player_left_add2 = {
			type = "input",
			name = "Add player smash left 2",
			disabled = disabled,
			order = 222,
		},
		p3_header = {
			type = "header",
			name = L.phase:format(3),
			order = 300,
		},
		bombsenable = {
			type = "toggle",
			name = L.custom_bombs,
			disabled = disabled,
			order = 301,
		},
		markedenable = {
			type = "toggle",
			name = L.custom_marks,
			disabled = disabled,
			order = 311,
		},
	},
}

SLASH_BigWigs_BlackhandAssist1 = "/bwblackhand"
SlashCmdList.BigWigs_BlackhandAssist = function()
	InterfaceOptionsFrame_OpenToCategory("Big Wigs Blackhand Assist")
	InterfaceOptionsFrame_OpenToCategory("Big Wigs Blackhand Assist")
end

-------------------------------------------------------------------------------
-- Event Handlers
--

function mod:OnBossEnable()
	--addon.db = LibStub("AceDB-3.0"):New("BlackhandAssistDB", defaults, true)
	--LibStub("AceConfig-3.0"):RegisterOptionsTable(L.modname, options)
	--LibStub("AceConfigDialog-3.0"):AddToBlizOptions(L.modname, L.modname)

	self:Log("SPELL_AURA_APPLIED", "AttachSlagBombs", 157000, 159179)
	self:Log("SPELL_AURA_APPLIED", "MarkedForDeathApplied", 156096)
	self:Log("SPELL_CAST_SUCCESS", "Mortar", 156530)

	self:Death("Disable", 77325)
end

function mod:OnEngage()
	smashCount = 1
	phase = 1
	markOnMe = nil
	mortarWarnings = {}
	self:RegisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", nil, "boss1")
end

function mod:OnBossDisable()
	--self:UnregisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", "boss1")
	--self:UnregisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", "boss2", "boss3", "boss4", "boss5")
	--self:UnregisterUnitEvent("UNIT_POWER_FREQUENT", "boss2", "boss3", "boss4", "boss5")
end

--------------------------------------------------------------------------------
-- Event Handlers
--
function mod:UNIT_SPELLCAST_SUCCEEDED(unit, spellName, _, _, spellId)
	if spellId == 158021 then -- Marked for Death Clear, before running to middle of room, starting p2
		if phase == 1 then
			phase = 2
			smashCount = 1
			self:RegisterUnitEvent("UNIT_POWER_FREQUENT", nil, "boss2", "boss3", "boss4", "boss5")
			self:RegisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", nil, "boss2", "boss3", "boss4", "boss5")
			--self:Log("SPELL_CAST_START", "ShatteringSmash", 155992, 159142)
		end
	elseif spellId == 156425 then -- Demolition
		self:ScheduleTimer("Message", 15, spellId, "Urgent", "Alert", CL.over:format(self:SpellName(156479)))
	elseif spellId == 161348 then -- Jump To Third Floor
		phase = 3
		self:UnregisterUnitEvent("UNIT_POWER_FREQUENT", "boss2", "boss3", "boss4", "boss5")
	end
end

function mod:UNIT_POWER_FREQUENT(unit)
	local power = UnitPower(unit)
	local guid = UnitGUID(unit)

	if power > 80 and not mortarWarnings[guid] then
		self:Message(156530, "Attention", "Alert", CL.soon:format(self:SpellName(156530)))
		mortarWarnings[guid] = true
	end
end

function mod:ShatteringSmash(args)
	smashCount = smashCount + 1
end

function mod:Mortar(args)
	self:Message(args.spellId, "Attention", "Info")
	mortarWarnings[args.sourceGUID] = nil
end

--------------------------------------------------------------------------------
-- Mark say
--

do
	local scheduled, markCount = nil, 0
	local function markedSay(self, spellId)
		--self:Message(spellId, list, phase == 3 and "Important" or "Attention", "Alarm", nil, nil, phase == 3)
		scheduled = nil

		local unit = "raid%d"
		local marks = 0
		for i = 1,30 do
			if UnitDebuff(unit:format(i), mod:SpellName(spellId)) then
				marks = marks + 1
				if UnitIsUnit(unit:format(i), "player") then
					if marks == 1 then --left
						mod:Say(spellId, "< left <", true)
						myMarkPosition = "left"
					elseif marks == 3 then --right
						mod:Say(spellId, "> right >", true)
						myMarkPosition = "right"
					else --center
						mod:Say(spellId, "|| center ||", true)
						myMarkPosition = "center"
					end
					break
				end
			end
		end
	end

	function mod:MarkedForDeathApplied(args)
		if markCount >= 3 then	-- new marks: reset
			markCount = 0
			markOnMe = false
		end

		markCount = markCount + 1

		if self:Me(args.destGUID) then
			markOnMe = true
		end

		if markCount == 3 and markOnMe then
			if not scheduled then
				scheduled = self:ScheduleTimer(markedSay, 0.2, self, args.spellId)
			end
		end
	end
end


--------------------------------------------------------------------------------
-- Bomb say
--

do
	local list, scheduled, bombCount = {}, nil, 0
	local function bombSay(self, spellId)
		--self:Message(spellId, list, phase == 3 and "Important" or "Attention", "Alarm", nil, nil, phase == 3)
		scheduled = nil

		local meleeCount, meleeName = 0, nil
		for _,v in pairs(list) do
			if mod:Damager(v) == "MELEE" then
				meleeCount = meleeCount + 1
				meleeName = v
			end
		end

		if meleeCount == 1 then	-- 1 Melee, 1 Ranged
			if UnitName("player") == meleeName then -- We are the melee
				mod:Say(spellId, "Bombs MIDDLE", true)
			else -- We are the Ranged
				mod:Say(spellId, "Bombs BACK", true)
			end
			return
		end

		local unit = "raid%d"
		local bombs = 0
		for i = 1,30 do
			if not mod:Tank(unit:format(i)) and UnitDebuff(unit:format(i), mod:SpellName(spellId)) then
				bombs = bombs + 1
				if UnitIsUnit(unit:format(i), "player") then
					if bombs == 1 then --middle
						mod:Say(spellId, "Bombs MIDDLE", true)
						myBombPosition = "middle"
					else --back
						mod:Say(spellId, "Bombs BACK", true)
						myBombPosition = "back"
					end
					break
				end
			end
		end
	end

	function mod:AttachSlagBombs(args)
		if bombCount >= 3 then	-- new bombs: reset
			bombCount = 0
			bombOnMe = false
			wipe(list)
		end

		bombCount = bombCount + 1

		if not self:Tank(args.destName) then
			list[#list+1] = args.destName
		end

		if self:Me(args.destGUID) then
			bombOnMe = true
		end

		if bombCount == 3 and bombOnMe then
			if not scheduled then
				scheduled = self:ScheduleTimer(bombSay, 0.2, self, args.spellId)
			end
		end
	end
end

--------------------------------------------------------------------------------
-- API
--

function addon:MarkPositon()
	return myMarkPosition
end

function addon:BombPositon()
	return myBombPosition
end